
import java.util.Vector;
import java.util.HashSet;
import java.util.HashMap;

public class NLFormula extends NLPhrase {
  
  boolean asserted,main;
  NLFormula f;
  String[] cc = new String[0];
  NLFormula[] ff;
  //Use these ones in translation.
  //They are set up by the makeMultiRelations method.
    NLFormula[] forms;
    String[] conns;
  //
  HashMap m;
  int parenDepth = 0;
  boolean isInWords = false;
  String translation = "";
  
  boolean diag = false;
  
  public NLFormula() {}
  
  public NLFormula(boolean asserted, boolean main, NLFormula f,
                   String[] cc, NLFormula[] ff) {
    //Import the fields.
    this.asserted = asserted; this.main = main; this.f = f;
    this.cc = cc; this.ff = ff;
    //Set up the phrase map.
    m = new HashMap();
    m.put("\\wedge","and");
    m.put("\\vee","or");
    m.put("\\rightleftarrow","if and only if");
    m.put("\\rightarrow","then");
    //Make any multi-relations, and prepare the 'forms' and 'conns' arrays.
    makeMultiRelations();
    //simpleFormsAndConns();
  }
  
  void simpleFormsAndConns() {
    forms = new NLFormula[ff.length+1];
    forms[0] = f;
    System.arraycopy(ff,0,forms,1,ff.length);
    conns = cc;
  }

  void makeMultiRelations() {
  //Combine any adjacent repeated assertions into MultiRelations.
  //We assume cc and ff have the same length.
    if (cc!=null) {
      //Put f and ff together, for uniformity.
      NLFormula[] formulae = new NLFormula[ff.length+1];
      formulae[0] = f;
      System.arraycopy(ff,0,formulae,1,ff.length);
//diag:
  if (diag) {
    String[] ss = new String[formulae.length];
    for (int i = 0; i < formulae.length; i++) {
      ss[i] = formulae[i].xlat(NLPhrase.DEFAULT_MODE);
    }
    System.out.printf("makeMultiRelations for%n");
    for (int i = 0; i < formulae.length; i++) {
      System.out.println("  "+ss[i]);
      if (i<formulae.length-1) {System.out.println("    "+cc[i]);}
    }
  }
//      
      //Now make a vector of int pairs, telling what ranges in temp,
      //if any, should be combined together.
      Vector vect = new Vector();
      int[] range = new int[2];
      NLRelation relA, relB;
      String a,b,commonReln = "";
      boolean assrtA, assrtB, commonAssrt = true;
      boolean withinASequence = false;
      for (int i = 0; i < cc.length; i++) {
        if (!withinASequence) {
        //We are not within a sequence yet. Check if one begins now.
          if (cc[i].equals("\\wedge") &&
              (formulae[i] instanceof NLRelation) &&
              (formulae[i+1] instanceof NLRelation)
          ) {
            relA = (NLRelation) formulae[i];
            relB = (NLRelation) formulae[i+1];
            a = relA.getRstr();
            b = relB.getRstr();
            assrtA = relA.getAsserted();
            assrtB = relB.getAsserted();
            if (a.equals(b) && assrtA==assrtB) {
            //It is time to begin a sequence.
              range[0] = i;
              withinASequence = true;
              commonReln = new String(a);
              commonAssrt = assrtA;
              if (i==cc.length-1) {
              //We are already at the end of the sequence.
                range[1] = i+1;
                withinASequence = false;
                int[] thisRange = new int[2];
                System.arraycopy(range,0,thisRange,0,2);
                vect.add(thisRange);
              }
            }
          }
        } else {
        //We are within a sequence. Check if it persists.
          boolean stillInSequence = true;
          stillInSequence &= (formulae[i+1] instanceof NLRelation);
          if (stillInSequence) {
            stillInSequence &= cc[i].equals("\\wedge");
            relB = (NLRelation) formulae[i+1];
            stillInSequence &= commonReln.equals(relB.getRstr());
            stillInSequence &= commonAssrt==relB.getAsserted();
          }
          if (!stillInSequence || i==cc.length-1) {
          //The sequence has ended.
            range[1] = i;
            withinASequence = false;
            int[] thisRange = new int[2];
            System.arraycopy(range,0,thisRange,0,2);
            vect.add(thisRange);
          }        
        }
      }
      
      //Now see what ranges we found.
      int[][] ranges = new int[0][];
      ranges = (int[][]) vect.toArray(ranges);
      if (ranges!=null && ranges.length > 0) {
      //We found one or more ranges.
//diag:
  if (diag) {
    System.out.printf("Found %d ranges.%n"+
      "The first range goes from %d to %d.%n",
      ranges.length,ranges[0][0],ranges[0][1]);
  }
//      
        //Start by figuring out the length of the forms and conns
        //arrays, and initializing them.
        int numberRemoved = 0;
        for (int[] r : ranges) {
          numberRemoved += (r[1]-r[0]);
        }
        forms = new NLFormula[formulae.length-numberRemoved];
        conns = new String[cc.length-numberRemoved];
        //Make an array to say which indices begin a range.
        boolean[] isStartIndex = new boolean[formulae.length];
        HashSet startIndices = new HashSet();
        for (int[] r : ranges) {
          startIndices.add(r[0]);
        }
        for (int i = 0; i < formulae.length; i++) {
          if (startIndices.contains(i)) {
            isStartIndex[i] = true;
          } else {
            isStartIndex[i]= false;
          }
        }
        //Make a map that sends a range start index to its end index.
        HashMap endIndices = new HashMap();
        for (int[] r : ranges) {
          endIndices.put(r[0],r[1]);
        }
        //Now populate the arrays.
        int ptr = 0; int i = 0;
        while (i < formulae.length) {
          if (!isStartIndex[i]) {
          //There is nothing to combine.
            forms[ptr] = formulae[i];
            if (i < cc.length) {
              conns[ptr] = cc[i];
            }
            ptr++; i++;
          } else {
          //It is time to do a combination.
            Integer endInt = (Integer) endIndices.get(i);
            int end = endInt.intValue();
            relA = (NLRelation) formulae[i];
            commonAssrt = relA.getAsserted();
            commonReln = relA.getRstr();
//diag:
  //System.out.printf("i is %d, and end is %d.%n",i,end);
//
            NLTerm[][] multiArgs = new NLTerm[end-i+1][];
            for (int j = i; j <= end; j++) {            
              relB = (NLRelation) formulae[j];
              multiArgs[j] = relB.getArgs();
            }
            NLMultiRelation multiRel = new NLMultiRelation(
              commonAssrt,commonReln,multiArgs
            );
            forms[ptr] = multiRel;
            if (end < cc.length) {
              conns[ptr] = cc[end];
            }
            ptr++; i = end+1;
          }
        }
        
      } else {
      //We found no ranges. Just make the forms array point to the
      //formulae array, and conns array point to cc.
        forms = formulae;
        conns = cc;
      }
    }
//diag:
  if (diag) {
    System.out.printf("%n");
  }
//      
  }

  void parenthesize() {
    parenDepth++;
  }

  public String xlat(int mode) {
    for (NLFormula form : forms) {
      form.xlat(mode); this.isInWords |= form.isInWords();
    }
    if (forms.length > 1 || !asserted) {this.isInWords = true;}
    String x = "";
    if (this.isInWords) {
      x = writeWords();
    } else {
      x = writeSymbols();
    }
    this.translation = x;
    return x;
  }
  
  public boolean isInWords() {
  //Should be called only *after* xlat.
    return this.isInWords;
  }

  public String getString() {
  //Should be called only *after* xlat.
    return this.translation;
  }
  
  String writeSymbols() {
  //We can get here only if 'forms' has length 1, and asserted is true.
    String s = "";
    s += forms[0].getString();
    //Parenthesize.
    String open = "",close = "";
    for (int i = 0; i < parenDepth; i++) {
      open += "("; close += ")";
    }
    s = open+s+close;
    return s;
  }
  
  String writeWords() {
    String s = "";
    if (!asserted) {s += "it is not the case that ";}
    s += imp(0);
    s += forms[0].getString();
    if (main && s.length()>0) {
      String init = s.substring(0,1);
      init = init.toUpperCase();
      s = init+s.substring(1,s.length());
    }
    for (int i = 0; i < conns.length; i++) {
      s += " "+m.get(conns[i])+" "+imp(i+1)+forms[i+1].getString();
    }
    return s;
  }
  
  private String imp(int i) {
    String x = "";
    if (conns.length>=i+1) {
      if (conns[i].equals("\\rightarrow")) {x = "if ";}
    }
    return x;
  }

}






