 

import java.util.HashMap;

public class NLTermformula extends NLFormula {

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------

  boolean asserted;
  String d;
  NLTerm t;
  NLTerm[] tt,uu;
  NLInfixrel[] aa;
  NLTerm[][] ttt;
  
  boolean isInWords = false;
  String translation = "";
  
  HashMap m;

// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public NLTermformula(boolean asserted, NLTerm t, String d,
                       NLTerm[] tt, NLTerm[] uu, NLInfixrel[] aa,
                       NLTerm[][] ttt) {
    this.asserted = asserted; this.t = t; this.d = d; this.tt = tt;
    this.uu = uu; this.aa = aa; this.ttt = ttt;
    
    m = new HashMap();
    m.put("\\uparrow","is undefined");
    m.put("\\downarrow","is defined");
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  public String xlat(int mode) {
    //First call the xlat methods of the children, and figure out
    //if we are forced to return words.
    //Start with the term t.
    t.xlat(mode); this.isInWords |= t.isInWords();
    //Check if we've asserted an up or down arrow.
    if (d!=null) {this.isInWords = true;}
    //Now the term array tt, if it is not null.
    if (tt!=null) {
      for (int i = 0; i < tt.length; i++) {
        tt[i].xlat(mode); this.isInWords |= tt[i].isInWords();
      }
    }
    //Now the terms uu, if they are there.
    boolean lastSet = t.isInWords();
    if (uu!=null) {
      for (NLTerm term : uu) {
        term.xlat(mode); lastSet |= term.isInWords();
      }
      this.isInWords |= lastSet;
    }
    //Now handle aa and ttt (we expect them to be the same length).
    if (aa!=null) {
      boolean nextSet = false;
      int lastLength = 1+(uu!=null?uu.length:0);
      for (int i = 0; i < aa.length; i++) {
        for (NLTerm term : ttt[i]) {
          term.xlat(mode); nextSet |= term.isInWords();
        }
        this.isInWords |= nextSet;
        if (lastSet || nextSet || mode==NLPhrase.WORDS_MODE) {
          if (lastLength==1) {
            aa[i].xlat(NLInfixrel.WORDS_SINGULAR_CODE);
          } else {
            aa[i].xlat(NLInfixrel.WORDS_PLURAL_CODE);
          }
        } else {
          aa[i].xlat(NLInfixrel.DEFAULT_CODE);
        }
        this.isInWords |= aa[i].isInWords();
        lastLength = ttt[i].length;
        lastSet = nextSet;
      }
    }    
    //Finally, write the string.
    String x = "";
    if (this.isInWords || mode==NLPhrase.WORDS_MODE) {
      x = writeWords();
    } else {
      x = writeSymbols();
    }
    this.translation = x;
    return x;
  }

  public boolean isInWords() {
  //Don't call until *after* calling xlat.
    return isInWords;
  }

  public String getString() {
  //Don't call until *after* calling xlat.
    return translation;
  }

  String writeSymbols() {
    String s = "";
        s += (asserted?"":"\\neg ");
        s += t.getString();
        if (d!=null) {
          s += " "+m.get(d);
        } else if (tt!=null) {
          s += "[";
          for (int i = 0; i < tt.length; i++) {
            s += (i==0?"":",")+tt[i].getString();
          }
          s += "]";
        } else {
          if (uu!=null) {
            for (int i = 0; i < uu.length; i++) {
              s += ","+uu[i].getString();
            }
          }
          for (int i = 0; i < aa.length; i++) {
            s += aa[i].getString();
            for (int j = 0; j < ttt[i].length; j++) {
              s += (j==0?"":",")+ttt[i][j].getString();
            }
          }
        }
    return s;
  }
  
  String writeWords() {
    String s = "";
        s += (asserted?"":"it is not the case that ");
        s += t.getString();
        if (d!=null) {
          s += " "+m.get(d);
        } else if (tt!=null) {
          s += ", regarded as a relation, asserted of the arguments ";
          for (int i = 0; i < tt.length; i++) {
            s += (i==0?"":", ")+(i==tt.length-1?"and ":"")+tt[i].getString();
          }
        } else {
          if (uu!=null) {
            for (int i = 0; i < uu.length; i++) {
              s += ", "+uu[i].getString();
            }
          }
          for (int i = 0; i < aa.length; i++) {
            s += aa[i].getString();
            for (int j = 0; j < ttt[i].length; j++) {
              s += (j==0?"":", ")+ttt[i][j].getString();
            }
          }
        }
    return s;
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 
